#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
/*
 * pptpctrl.c
 *
 * PPTP control connection between PAC-PNS pair
 * This is my hack at establishing and controling the PPTP CONTROL
 * connection.. ctrlpacket.c handles the packet encoding/decoding
 *
 * Matthew Ramsay <matthewr@moreton.com.au>
 *
 */


#include <netinet/in.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <unistd.h>

#include "inststr.h"
#include "pptpctrl.h"
#include "pptpdebug.h"
#include "pptpdefs.h"
#include "ctrlpacket.h"


static int stop_bludging = 0;

void bludge(int sig) {

    stop_bludging = 1;

}


// local function prototypes
int pptp_establish_ctrl_connection(int clientFd, int pid);



/*
 * pptp_open_ctrl_connection
 *
 * Listens for connections on port 1723.. it loops 4ever waiting for a connection.
 *
 *
 * retn: fd on success, -1 when somethin real bad happens :)
 *
 */
int pptp_open_ctrl_connection(int argc, char **argv, char **envp, int pipe_fd[2], int ctrl_pid) {

	pid_t pid, parent_pid;

        int serverFd; // server socket
        int clientFd; // client socket

        struct sockaddr_in serverINETAddress; // server address
        struct sockaddr_in clientINETAddress; // client address
        int addrlen;

        struct in_addr inetaddr;

        // debugging
        char debug_msg[MAX_DEBUG_MSG_SIZE];
        unsigned char foobar[4];
	char buf[32]; // for changing the process name



	if((serverFd = socket(AF_INET, SOCK_STREAM, 0)) == -1) {
	
		// oops! couldn't create server socket
		pptpd_debug("CTRL: couldn't create server socket", CRITICAL);
		return -1;
		
	}
	
	addrlen = sizeof(serverINETAddress);
	
	bzero((char *) &serverINETAddress, addrlen);
	
	serverINETAddress.sin_family = AF_INET;
	serverINETAddress.sin_addr.s_addr = INADDR_ANY;
	serverINETAddress.sin_port = htons(PPTP_PORT);
	
	if((bind(serverFd, (struct sockaddr*) &serverINETAddress, addrlen)) == -1) {
	
		// oops! couldn't bind address
		pptpd_debug("CTRL: couldn't bind address", CRITICAL);
		perror("CTRL: couldn't bind");
		
		return -1;
		
	}
	 	
	
	if((listen(serverFd, MAX_PENDING_CONNECTIONS)) == -1) {
	
		// oops! couldn't listen
		pptpd_debug("CTRL: couldn't listen", CRITICAL);
		perror("CTRL: couldn't listen");
		
		return -1;
		
	}
	
	
	sprintf(debug_msg, "CTRL: CTRL connection listening on port %d", PPTP_PORT);
	pptpd_debug(debug_msg, VERBOSE);
	
	signal(SIGUSR2, bludge);
	parent_pid = getpid();
	pid = fork();
	if(pid > 0) {
	
		// parent

		pptpd_debug("CTRL: Waiting for the connecting clients INETADDR to be resolved", VERBOSE);		
		if(!stop_bludging) {
		
			pause();
			
		}
		pptpd_debug("CTRL: connected clients INETADDR resolved.. returning", VERBOSE);
		
		return serverFd;
		
	} else {

		//signal(SIGUSR1, SIG_DFL);
		
		// let's give the process a new name
		strcpy(buf, "PPTP: CTRL Connection");
		inststr(argc, argv, envp, buf);
		
		
		// child
		//////////////////////////////	
		while(1) { // loop forever.. waiting for connections
	
			pptpd_debug("CTRL: waiting for connection...", VERBOSE);
			
			// wait for a connection	
			if((clientFd = accept(serverFd, (struct sockaddr*) &clientINETAddress, &addrlen)) == -1) {
				// couldn't accept the clients connection
				pptpd_debug("CTRL: wouldn't accept() connection attempt", CRITICAL);
				return -1;
		
			} else {

				// get the clients INETADDR		
				memcpy(&inetaddr, &clientINETAddress.sin_addr, 4);
				
				memcpy(foobar, &inetaddr, 4);
				pptpd_byte_debug("CTRL: client inetaddr", foobar, 4);
				
				// interprocess communication
				// tell the GRE fork what the clients INETADDR is
				close(pipe_fd[PIPE_READ]);
				write(pipe_fd[PIPE_WRITE], foobar, 4);
				close(pipe_fd[PIPE_WRITE]);
				///////////////////////////////////////////
				
		
				pptpd_debug("CTRL: about to send a signal to SIGUSR2", VERBOSE);
				kill(parent_pid, SIGUSR2);
						
				pptpd_debug("CTRL: connection established!", VERBOSE);
				pptpd_debug("CTRL: attempting to handle PPTP control connection", VERBOSE);
		
				pptp_establish_ctrl_connection(clientFd, ctrl_pid);
				
			}
			
			close(clientFd);
	
		}
	
	}
	
	return -1; // shouldn't get out ere...
	
}


///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
// local functions only below
///////////////////////////////////////////////////////////////////////////

/*
 * pptp_establish_ctrl_connection
 *
 * 1. read a packet (should be start_ctrl_conn_rqst)
 * 2. reply to packet (send a start_ctrl_conn_rply)
 * 3. ...
 *
 *
 * args: clientFd - client socket
 * retn: 0 success, -1 failure
 *
 */
int pptp_establish_ctrl_connection(int clientFd, int pid) {

	int i;

        size_t rply_size; // reply packet size

        fd_set rfds;
        int stat1, stat2; // @stolen
        	
	unsigned char packet[PPTP_MAX_CTRL_PCKT_SIZE];
	unsigned char rply_packet[PPTP_MAX_CTRL_PCKT_SIZE];

        // debugging
	char debug_msg[MAX_DEBUG_MSG_SIZE];
	
	
	FD_ZERO(&rfds);
	FD_SET(clientFd, &rfds);
		
	// this concept first noted in pptp_gre by Ananian
	// i'll apply it here too. :-)
	// gotta put this back to normal one day soon
	while(1/*stat1 >= 0 && stat2 >= 0*/) { // keep looping until a read/write fails
		
		//stat =
		stat1 = read_pptp_packet(clientFd, packet, rply_packet, &rply_size, pid);
		
		pptpd_debug("CTRL: about reply to the packet we just got", VERBOSE);
		
		// stat =
		stat2 = send_pptp_packet(clientFd, rply_packet, rply_size);
		
		sprintf(debug_msg, "stat1 (read) = %d, stat2 (send) = %d", stat1, stat2);
		pptpd_debug(debug_msg, VERBOSE);
		
	}
	
	pptpd_debug("CTRL: we got kicked out of the STAT while loop", VERBOSE);

	return 0;
		
}

