#ifdef HAVE_CONFIG_H
#include "config.h"
#endif
/*
 * pptpdebug.c
 *
 * PPTP debug message formatting
 * Here is my hack at making some kind of debugging log file.
 * Probably not the best way to do this.. but I was trying to make
 * my log files look half useful.
 *
 *
 * Matthew Ramsay <matthewr@moreton.com.au>
 *
 */


#include <string.h>
#include <time.h>

#include "pptpdebug.h"


// local function prototypes
FILE *pptpd_open_logfile();
void pptp_close_logfile(FILE *in);
void pptpd_log_msg(FILE *in, char *debug_msg);
void pptp_log_time(FILE *in);
void pptp_print_bytes(char *debug_msg);



/*
 * pptpd_debug
 *
 * Display a PPTP SERVER message according to the debugging level
 * specified in pptp_debug.h
 *
 *
 * args: debug_msg - debug message
 *	 level - either VERBOSE or CRITICAL
 *
 */
void pptpd_debug(char *debug_msg, int level) {

	// this is my attempt at making a half decent way of
	// presenting debugging information
	
	FILE *in;
	
	if(PPTP_DEBUGGING) { // check if it is ok to print debug messages
	
		in = pptpd_open_logfile();
					
		if(PPTP_DEBUGGING_LEVEL == VERBOSE)
			// display all messages
			pptpd_log_msg(in, debug_msg);
			
		
		else if(PPTP_DEBUGGING_LEVEL == CRITICAL) {
		
			if(level == VERBOSE)
				//do nothing
				;
			
			else if(level == CRITICAL)
				// display any CRITICAL message
				pptpd_log_msg(in, debug_msg);
		
		}
		
		pptp_close_logfile(in);
	
	}

} // pptpd_debug


/*
 * pptpd_byte_debug
 *
 * Write a byte dump to the log file
 *
 * args: description - byte dump name (eg. "GRE packet")
 * 	 buffer - array of bytes to dump to log file
 *	 bufsize - number of bytes to display in buffer
 *
 */
void pptpd_byte_debug(unsigned char *description, unsigned char *buffer, int bufsize) {

	int i;
        char debug_msg[MAX_DEBUG_MSG_SIZE];

        sprintf(debug_msg, "***** %s BYTE DUMP: ", description);
        pptpd_debug(debug_msg, VERBOSE);
	
	for(i = 0; i < bufsize; i++) {
		
		sprintf(debug_msg, "%02x", buffer[i]);
		pptp_print_bytes(debug_msg);

	}
	
	//pptpd_debug("***** END OF BYTE DUMP *****", VERBOSE);

} // pptpd_byte_debug



///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////
// LOCAL functions only below
///////////////////////////////////////////////////////////////////////////

/*
 * pptp_print_bytes
 *
 * Prints out bytes in debug VERBOSE mode only
 *
 * args: debug_msg - bytes to display
 *
 */
void pptp_print_bytes(char *debug_msg) {

	FILE *in;
	

	if(PPTP_DEBUGGING) { // check if it is ok to print debug messages
	
		if(PPTP_DEBUGGING_LEVEL == VERBOSE) { // only in VERBOSE mode
		
			in = pptpd_open_logfile();
			fprintf(in, "%s", debug_msg);
			pptp_close_logfile(in);
		
		}
	
	}

} // pptp_print_bytes


/*
 * pptpd_log_msg
 *
 * Simply appends the log time and default message to the beginning of the debug message
 *
 * args: in - the log FILE pointer
 *	 debug_msg - the message to display
 *
 */
void pptpd_log_msg(FILE *in, char *debug_msg) {

	pptp_log_time(in);
	fprintf(in, "PPTPD %s", debug_msg);

} // pptpd_log_msg


/*
 * pptpd_open_logfile
 *
 * Opens the PPTPD log file for appending/creating
 *
 * retn: FILE pointer
 *
 */
FILE *pptpd_open_logfile() {

	FILE *in;
	char filename[PPTPD_LOG_FILENAME_SIZE];

	
	strcpy(filename, PPTPD_LOG_PATH);
	strcat(filename, PPTPD_LOG_FILENAME);
		
	if((in = fopen(filename, "a")) == NULL) {
		
		// oops.. couldn't open/create the log file
		perror("PPTPD log");
		return NULL;
		
	}
	
	return in;
		
} // pptpd_open_logfile


/*
 * pptp_close_logfile
 *
 * Closes a PPTP log file
 *
 * args: in - FILE pointer to close
 *
 */
void pptp_close_logfile(FILE *in) {

	fclose(in);
	
} // pptp_close_logfile


/*
 * pptp_log_time
 *
 * Writes the current localtime to the FILE
 *
 * args: in - FILE pointer
 *
 */
void pptp_log_time(FILE *in) {

	struct tm *log_time;
	time_t t;
	
	t = time(NULL);
	log_time = localtime(&t);
	
	fprintf(in, "\n[%02d:%02d:%02d %02d/%02d/%02d] ", log_time->tm_hour, log_time->tm_min,
		log_time->tm_sec, log_time->tm_mday, log_time->tm_mon, log_time->tm_year);
	
} // pptp_log_time


